@@ -86,6 +86,7 @@ extern char *optarg;
 #endif
 
 #include "iso9660.h"
+#include "defaults.h"
 
 struct directory_entry{
   struct directory_entry * next;
@@ -166,7 +167,7 @@ extern void DECL(assign_directory_addresses,(struct directory * root));
 
 extern int DECL(iso9660_file_length,(const char* name, 
 			       struct directory_entry * sresult, int flag));
-extern int DECL(iso_write,(FILE * outfile, char * volid));
+extern int DECL(iso_write,(FILE * outfile));
 extern void generate_path_tables();
 extern void DECL(generate_iso9660_directories,(struct directory *, FILE*));
 extern void DECL(generate_one_directory,(struct directory *, FILE*));
@@ -205,6 +206,13 @@ extern char * publisher;
 extern char * copyright;
 extern char * biblio;
 extern char * abstract;
+extern char * appid;
+extern char * volset_id;
+extern char * system_id;
+extern char * volume_id;
+
+extern void * DECL(e_malloc,(size_t));
+
 
 #define SECTOR_SIZE (2048)
 #define ROUND_UP(X)    ((X + (SECTOR_SIZE - 1)) & ~(SECTOR_SIZE - 1))
