@@ -27,7 +27,7 @@
 
 #ifndef VMS
 #include <unistd.h>
-#if	!defined(__QNX__) && !defined(__NetBSD__)
+#ifdef HASSYSMACROS
 #include <sys/sysmacros.h>
 #endif
 #else
@@ -189,7 +189,7 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
 		  if(s_entry->table) tablesize += 35 + strlen(s_entry->table);
 	  };
 		  table = (struct directory_entry *) 
-		malloc(sizeof (struct directory_entry));
+		e_malloc(sizeof (struct directory_entry));
  	memset(table, 0, sizeof(struct directory_entry));
 	table->table = NULL;
 	table->next = this_dir->contents;
@@ -206,7 +206,7 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
 	table->size = tablesize;
 	table->filedir = this_dir;
 	table->name = strdup("<translation table>");
-	table->table = (char *) malloc(ROUND_UP(tablesize));
+	table->table = (char *) e_malloc(ROUND_UP(tablesize));
 	memset(table->table, 0, ROUND_UP(tablesize));
 	iso9660_file_length  ("TRANS.TBL", table, 1);
 		
@@ -334,7 +334,7 @@ static void generate_reloc_directory()
 	/* Create an  entry for our internal tree */
 	time (&current_time);
 	reloc_dir = (struct directory *) 
-		malloc(sizeof(struct directory));
+		e_malloc(sizeof(struct directory));
 	memset(reloc_dir, 0, sizeof(struct directory));
 	reloc_dir->parent = root;
 	reloc_dir->next = root->subdir;
@@ -348,7 +348,7 @@ static void generate_reloc_directory()
 	
 	/* Now create an actual directory  entry */
 	s_entry = (struct directory_entry *) 
-		malloc(sizeof (struct directory_entry));
+		e_malloc(sizeof (struct directory_entry));
 	memset(s_entry, 0, sizeof(struct directory_entry));
 	s_entry->next = root->contents;
 	reloc_dir->self = s_entry;
@@ -375,7 +375,7 @@ static void generate_reloc_directory()
 	/* Now create the . and .. entries in rr_moved */
 	/* Now create an actual directory  entry */
 	s_entry = (struct directory_entry *) 
-		malloc(sizeof (struct directory_entry));
+		e_malloc(sizeof (struct directory_entry));
 	memcpy(s_entry, root->contents, 
 	       sizeof(struct directory_entry));
 	s_entry->name = strdup(".");
@@ -393,7 +393,7 @@ static void generate_reloc_directory()
 	};
 	
 	s_entry = (struct directory_entry *) 
-		malloc(sizeof (struct directory_entry));
+		e_malloc(sizeof (struct directory_entry));
 	memcpy(s_entry, root->contents, 
 	       sizeof(struct directory_entry));
 	s_entry->name = strdup("..");
@@ -482,15 +482,23 @@ FDECL2(scan_directory_tree,char *, path, struct directory_entry *, de){
   struct directory_entry  *s_entry, *s_entry1;
   struct directory * this_dir, *next_brother, *parent;
   struct stat statbuf, lstatbuf;
-  int status;
+  int status, dflag;
   char * cpnt;
   int new_reclen;
   int deep_flag;
 
   current_dir = opendir(path);
-  if(!current_dir) {
+  d_entry = NULL;
+
+  /* Apparently NFS sometimes allows you to open the directory, but
+     then refuses to allow you to read the contents.  Allow for this */
+
+  if(current_dir) d_entry = readdir(current_dir);
+
+  if(!current_dir || !d_entry) {
 	  fprintf(stderr,"Unable to open directory %s\n", path);
 	  de->isorec.flags[0] &= ~2; /* Mark as not a directory */
+	  if(current_dir) closedir(current_dir);
 	  return 0;
   };
 
@@ -502,7 +510,7 @@ FDECL2(scan_directory_tree,char *, path, struct directory_entry *, de){
   vms_path_fixup(path);
 #endif
 
-  this_dir = (struct directory *) malloc(sizeof(struct directory));
+  this_dir = (struct directory *) e_malloc(sizeof(struct directory));
   this_dir->next = NULL;
   new_reclen = 0;
   this_dir->subdir = NULL;
@@ -558,9 +566,14 @@ FDECL2(scan_directory_tree,char *, path, struct directory_entry *, de){
 
 /* Now we scan the directory itself, and look at what is inside of it. */
 
+  dflag = 0;
   while(1==1){
 
-    d_entry = readdir(current_dir);
+    /* The first time through, skip this, since we already asked for
+       the first entry when we opened the directory. */
+    if(dflag) d_entry = readdir(current_dir);
+    dflag++;
+
     if(!d_entry) break;
 
     /* OK, got a valid entry */
@@ -700,7 +713,7 @@ FDECL2(scan_directory_tree,char *, path, struct directory_entry *, de){
     };
 
     s_entry = (struct directory_entry *) 
-      malloc(sizeof (struct directory_entry));
+      e_malloc(sizeof (struct directory_entry));
     s_entry->next = this_dir->contents;
     this_dir->contents = s_entry;
     deep_flag = 0;
@@ -746,7 +759,7 @@ FDECL2(scan_directory_tree,char *, path, struct directory_entry *, de){
 		  if(!reloc_dir) generate_reloc_directory();
 
 		  s_entry1 = (struct directory_entry *) 
-			  malloc(sizeof (struct directory_entry));
+			  e_malloc(sizeof (struct directory_entry));
 		  memcpy(s_entry1, this_dir->contents, 
 			 sizeof(struct directory_entry));
 		  s_entry1->table = NULL;
