@@ -98,7 +98,7 @@ void FDECL2(set_733, char *, pnt, unsigned int, i){
 	pnt[4] = pnt[3] = (i >> 24) &  0xff;
 }
 
-static xfwrite(void * buffer, int count, int size, FILE * file)
+static FDECL4(xfwrite, void *, buffer, int, count, int, size, FILE *, file)
 {
   if (!fwrite(buffer, count, size, file) != size) return;
   fprintf(stderr,"cannot write raw file\n"); 
@@ -250,7 +250,7 @@ void FDECL1(sort_directory, struct directory_entry **, sort_dir){
   /* OK, now we know how many there are.  Build a vector for sorting. */
 
   sortlist =   (struct directory_entry **) 
-    malloc(sizeof(struct directory_entry *) * dcount);
+    e_malloc(sizeof(struct directory_entry *) * dcount);
 
   dcount = 0;
   s_entry = *sort_dir;
@@ -360,7 +360,7 @@ static void FDECL1(assign_file_addresses, struct directory *, dpnt){
 	
 	  if(s_entry->size) {
 	    dwpnt = (struct deferred_write *) 
-	      malloc(sizeof(struct deferred_write));
+	      e_malloc(sizeof(struct deferred_write));
 	    if(dw_tail){
 	      dw_tail->next = dwpnt;
 	      dw_tail = dwpnt;
@@ -424,7 +424,7 @@ void FDECL2(generate_one_directory, struct directory *, dpnt, FILE *, outfile){
   unsigned int dir_index, ce_index;
 
   total_size = (dpnt->size + (SECTOR_SIZE - 1)) &  ~(SECTOR_SIZE - 1);
-  directory_buffer = (char *) malloc(total_size);
+  directory_buffer = (char *) e_malloc(total_size);
   memset(directory_buffer, 0, total_size);
   dir_index = 0;
 
@@ -432,7 +432,7 @@ void FDECL2(generate_one_directory, struct directory *, dpnt, FILE *, outfile){
   ce_buffer = NULL;
 
   if(ce_size) {
-    ce_buffer = (char *) malloc(ce_size);
+    ce_buffer = (char *) e_malloc(ce_size);
     memset(ce_buffer, 0, ce_size);
     
     ce_index = 0;
@@ -559,14 +559,14 @@ void generate_path_tables(){
   /* First allocate memory for the tables and initialize the memory */
 
   tablesize = path_blocks << 11;
-  path_table_m = (char *) malloc(tablesize);
-  path_table_l = (char *) malloc(tablesize);
+  path_table_m = (char *) e_malloc(tablesize);
+  path_table_l = (char *) e_malloc(tablesize);
   memset(path_table_l, 0, tablesize);
   memset(path_table_m, 0, tablesize);
 
   /* Now start filling in the path tables.  Start with root directory */
   path_table_index = 0;
-  pathlist = (struct directory **) malloc(sizeof(struct directory *) * next_path_index);
+  pathlist = (struct directory **) e_malloc(sizeof(struct directory *) * next_path_index);
   memset(pathlist, 0, sizeof(struct directory *) * next_path_index);
   build_pathlist(root);
 
@@ -620,7 +620,7 @@ void generate_path_tables(){
 	    path_table_size);
 }
 
-int FDECL2(iso_write, FILE *, outfile, char *, volid){
+int FDECL1(iso_write, FILE *, outfile){
   char buffer[2048];
   char iso_time[17];
   int should_write;
@@ -650,14 +650,10 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
   vol_desc.version[0] = 1;
 
   memset(vol_desc.system_id, ' ', sizeof(vol_desc.system_id));
-#ifdef __QNX__
-  memcpy(vol_desc.system_id, "QNX", sizeof("QNX"));
-#else
-  memcpy(vol_desc.system_id, "LINUX", sizeof("LINUX"));
-#endif
+  memcpy(vol_desc.system_id, system_id, strlen(system_id));
 
   memset(vol_desc.volume_id, ' ', sizeof(vol_desc.volume_id));
-  memcpy(vol_desc.volume_id, volid, strlen(volid));
+  memcpy(vol_desc.volume_id, volume_id, strlen(volume_id));
 
   should_write = last_extent;
   set_733(vol_desc.volume_space_size, last_extent);
@@ -683,14 +679,26 @@ int FDECL2(iso_write, FILE *, outfile, char *, volid){
      though */
 
   FILL_SPACE(volume_set_id);
+  if(volset_id)  memcpy(vol_desc.volume_set_id,  volset_id, strlen(volset_id));
+
   FILL_SPACE(publisher_id);
   if(publisher)  memcpy(vol_desc.publisher_id,  publisher, strlen(publisher));
+
   FILL_SPACE(preparer_id);
   if(preparer)  memcpy(vol_desc.preparer_id,  preparer, strlen(preparer));
+
   FILL_SPACE(application_id);
+  if(appid) memcpy(vol_desc.application_id, appid, strlen(appid));
+
   FILL_SPACE(copyright_file_id);
+  if(appid) memcpy(vol_desc.copyright_file_id, appid, strlen(appid));
+
   FILL_SPACE(abstract_file_id);
+  if(appid) memcpy(vol_desc.abstract_file_id, appid, strlen(appid));
+
   FILL_SPACE(bibliographic_file_id);
+  if(appid) memcpy(vol_desc.bibliographic_file_id, appid, strlen(appid));
+
   FILL_SPACE(creation_date);
   FILL_SPACE(modification_date);
   FILL_SPACE(expiration_date);
